'use strict';

var test = require('tape');
var StyleDeclaration = require('../../../js/style/style_declaration');

test('StyleDeclaration', function(t) {
    t.test('boolean', function(t) {
        var decl = new StyleDeclaration({type: "boolean"}, false);
        t.equal(decl.calculate(0), false);
        t.end();
    });

    t.test('image', function(t) {
        var decl = new StyleDeclaration({type: "image", transition: true}, 'smilingclownstaringatyou.png');
        t.deepEqual(decl.calculate(0, { lastIntegerZoomTime: 0, lastIntegerZoom: 0 }, 300),
            { to: 'smilingclownstaringatyou.png', toScale: 1, from: 'smilingclownstaringatyou.png', fromScale: 0.5, t: 1 });
        t.end();
    });

    t.test('enum', function(t) {
        var decl = new StyleDeclaration({type: "enum"}, 'viewport');
        t.equal(decl.calculate(0), 'viewport');
        t.end();
    });

    t.test('array', function(t) {
        var decl = new StyleDeclaration({type: "array", transition: true}, [0, 10, 5]);
        t.deepEqual(decl.calculate(0, { lastIntegerZoomTime: 0, lastIntegerZoom: 0 }, 300),
            { to: [ 0, 10, 5 ], toScale: 1, from: [ 0, 10, 5 ], fromScale: 0.5, t: 1 });
        t.end();
    });

    t.test('constant', function(t) {
        t.equal((new StyleDeclaration({type: "number"}, 5)).calculate(0), 5);
        t.equal((new StyleDeclaration({type: "number"}, 5)).calculate(100), 5);
        t.end();
    });

    t.test('functions', function(t) {
        var reference = {type: "number", function: "interpolated"};
        t.equal((new StyleDeclaration(reference, { stops: [] })).calculate(0), 1);
        t.equal((new StyleDeclaration(reference, { stops: [[2, 2], [5, 10]] })).calculate(0), 2);
        t.equal((new StyleDeclaration(reference, { stops: [[0, 0], [5, 10]] })).calculate(12), 10);
        t.equal((new StyleDeclaration(reference, { stops: [[0, 0], [5, 10]] })).calculate(6), 10);
        t.equal(Math.round((new StyleDeclaration(reference, { stops: [[0, 0], [5, 10]], base: 1.01 })).calculate(2.5)), 5);
        t.equal((new StyleDeclaration(reference, { stops: [[0, 0], [1, 10], [2, 20]] })).calculate(2), 20);
        t.equal((new StyleDeclaration(reference, { stops: [[0, 0], [1, 10], [2, 20]] })).calculate(1), 10);
        t.equal((new StyleDeclaration(reference, { stops: [[0, 0]] })).calculate(6), 0);
        t.end();
    });

    t.test('color parsing', function(t) {
        var reference = {type: "color", function: "interpolated"};
        t.deepEqual(new StyleDeclaration(reference, 'red').calculate(0), [ 1, 0, 0, 1 ]);
        t.deepEqual(new StyleDeclaration(reference, '#ff00ff').calculate(0), [ 1, 0, 1, 1 ]);
        t.deepEqual(new StyleDeclaration(reference, { stops: [[0, '#f00'], [1, '#0f0']] }).calculate(0), [1, 0, 0, 1]);
        // cached
        t.deepEqual(new StyleDeclaration(reference, '#ff00ff').calculate(0), [ 1, 0, 1, 1 ]);
        t.deepEqual(new StyleDeclaration(reference, 'rgba(255, 51, 0, 1)').calculate(0), [ 1, 0.2, 0, 1 ]);
        t.end();
    });
});
